#ifndef VORBISDECODER_H
#define VORBISDECODER_H

/*
 *  vorbisdecoder.h
 *  OGG Vorbis decoding routines
 *
 *  Created by Alexander Thomas on 2011/01/15
 * Released under the New BSD License
 * Contact: doctor.lex at gmail.com
 *
 * Copyright (c) 2011, Alexander Thomas
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Alexander Thomas nor the
 *       names of contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <cstdio>

#define OV_EXCLUDE_STATIC_CALLBACKS

extern "C" {
#include <vorbis/codec.h>
#include <vorbis/vorbisfile.h>
}

#include "decoders.h"

class VorbisDecoder
{
public:
	/* Constructor. */
	VorbisDecoder();
	/* Shortcut constructor, equivalent to calling the init() method. */
	VorbisDecoder( char *fileName );
	~VorbisDecoder();

	/* Initialize the Decoder to read from a file.
	 * Return value indicates success. */
	bool init( char *fileName );

	/* Finish decoding and deallocate data.
	 * init() must be called again to re-use the decoder after having called finish(). */
	void finish();

	/* Return the number of channels. */
	unsigned int getNumChannels() const;
	/* Return the sample rate. */
	long getSampleRate() const;

	/* Decode a chunk of nSamples.
	 * Data pointer(s) inside output must already be allocated as nSamples floats.
	 * The return value is true if any data was output. */
	bool getSamples( outBuffer &output, unsigned long nSamples );

	/* Print some status information to stdout */
	void printStatus() const;

private:
	/* Run the decoder until the output buffer contains at least nSamples.
	 * Return value indicates if an error occurred or not. */
	bool run( unsigned long nSamples=1 );

	/* Send decoded samples to output buffer. */
	void sendOutput( float **pcm, long nSamples );

	/* Print a Vorbis error message */
	void vorbis_error( int vorbis_error, const char* message, ... ) const;

	/*-- Data members --*/
	char *m_fileName;
	OggVorbis_File  m_vorbisFile;
	vorbis_info    *m_vorbisInfo;
	int             m_nVorbisSection;

	unsigned long m_nSamplesRequested;

	struct outBuffer   m_outBuffer;
	unsigned long      m_nOutBufferSize;
};

#endif /* VORBISDECODER_H */
